import WebsocketHandle from './lib/WebsocketHandle'
import util from './util/index'

class Debugger extends WebsocketHandle {
  constructor (websocket, runtime, debuggerIns) {
    super(websocket)
    this.runtime = runtime
    this.debugger = debuggerIns
    this.debuggerHandlers = {
      'Debugger.enable': this.handleEnableCommand,
      'Debugger.setPauseOnExceptions': this.handleSetPauseOnExceptions,
      'Debugger.setAsyncCallStackDepth': this.handleSetAsyncCallStackDepth,
      'Debugger.setBlackboxPatterns': this.handleSetBlackboxPatterns,
      'Debugger.getScriptSource': this.handleGetScriptSourceCommand,
      'Debugger.stepOver': this.handleStepOverCommand,
      'Debugger.stepInto': this.handleStepIntoCommand,
      'Debugger.stepOut': this.handleStepOutCommand,
      'Debugger.resume': this.handleResumeCommand,
      'Debugger.setSkipAllPauses': this.handleSetSkipAllPauses,
      'Debugger.continueToLocation': this.handleContinueToLocation,
      'Debugger.getPossibleBreakpoints': this.handleGetPossibleBreakpoints,
      'Debugger.setBreakpointByUrl': this.handleSetBreakpointByUrl,
      'Debugger.removeBreakpoint': this.handleRemoveBreakpoint,
      'Debugger.evaluateOnCallFrame': this.handleEvaluateOnCallFrame,
      'Debugger.setBreakpointsActive': this.handleSetBreakpointActive
    }
    this.fireSourceParsedEvent()
  }

  handle (e) {
    const t = e.id
    const r = e.method
    const n = e.params
    const i = this.debuggerHandlers[r]
    i ? i.call(this, t, n) : this.respond(t)
  }

  handleEnableCommand (e, t) {
    this.respond(e, {
      debuggerId: '0'
    })
  }

  handleSetPauseOnExceptions (e, t) {
    this.respond(e)
  }

  handleSetAsyncCallStackDepth (e, t) {
    this.respond(e)
  }

  handleSetBlackboxPatterns (e, t) {
    this.respond(e)
  }

  handleGetScriptSourceCommand (e, t) {
    // 临时测试使用
    this.respond(e, {
      result: {
        scriptSource: `Page({
          // 事件处理函数
          bindViewTap() {
            this.demo()
          },
          demo() {
            let a = 2
            setInterval(() => { a++; console.log(a); }, 1000)
            let b = { c: 'this is demo' }
            console.log(a, b)
          }
        })
        `
      }
    })
  }

  handleResumeCommand (e) {
    this.debugger.resume()
    this.respond(e)
  }

  handleSetSkipAllPauses (e) {
    // this.debugger.setSkipAllPauses()
    this.respond(e)
  }

  handleStepOverCommand (e) {
    this.debugger.stepOver()
    this.respond(e)
  }

  handleStepIntoCommand (e) {
    this.debugger.stepInto()
    this.respond(e)
  }

  handleStepOutCommand (e) {
    this.debugger.stepOut()
    this.respond(e)
  }

  handleContinueToLocation (e, t) {
    const r = t.location
    r.lineNumber && this.debugger.continueToLocation(+r.scriptId, r.lineNumber + 1, r.columnNumber)
    this.respond(e)
  }

  handleGetPossibleBreakpoints (e, t) {
    const r = t.start.lineNumber || 0
    const n = t.start.columnNumber || 0
    // const i = t.end ? t.end.lineNumber : 0
    // const a = t.end && t.end.columnNumber ? t.end.columnNumber : 0
    const s = t.start.scriptId
    const scriptLines = this.getScriptLinesByScripId(s)
    if (!scriptLines.includes(r + 1)) {
      return
    }
    const result = {
      locations: [
        {
          scriptId: s,
          lineNumber: r,
          columnNumber: n
        },
        {
          scriptId: s,
          lineNumber: r,
          columnNumber: n,
          type: 'call'
        }
      ]
    }
    this.debugger.setBreakpoint({
      scriptId: s,
      line: r + 1,
      start: t
    })
    this.respond(e, result)
  }

  handleSetBreakpointByUrl (e, t) {
    // 此处需要判断 t.url 是否存在
    let breakpointId = `${t.url}_${t.lineNumber}`
    const item = this.debugger.breakpoints[breakpointId]
    const h = {
      scriptId: this.getScriptIdByUrl(t.url),
      lineNumber: t.lineNumber,
      columnNumber: t.columnNumber
    }
    if (item) {
      breakpointId = item.breakpointId
    } else {
      this.debugger.breakpoints[breakpointId] = h
    }

    this.respond(e, {
      result: { breakpointId: breakpointId, locations: [h] }
    })
  }

  handleRemoveBreakpoint (e, t) {
    const r = t.breakpointId
    this.debugger.removeBreakpoint(r)
    this.respond(e, { result: {} })
  }

  handleEvaluateOnCallFrame (e, t) {
  }

  handleSetBreakpointActive (e, t) {
    this.respond(e, { result: {} })
  }

  fireSourceParsedEvent (e, t, r, n, i, a, s) {
    if (__wxConfig__ && __wxConfig__.__scriptIds__) {
      const __scriptIds__ = __wxConfig__.__scriptIds__
      for (const key in __scriptIds__) {
        const element = __scriptIds__[key]
        const o = {
          'scriptId': key,
          'url': `http://127.0.0.1/${element}`,
          'startLine': 0,
          'startColumn': 0,
          'endLine': 1,
          'endColumn': 34,
          'executionContextId': 20,
          'hash': "3df3dfbe82f1b8d573687dc431face680292668e",
          'executionContextAuxData': {
            'isDefault': true,
            'type': "default",
            'frameId': "368C0C1D68259DE66A5E387A418366F4"
          },
          'isLiveEdit': false,
          'sourceMapURL': "",
          'hasSourceURL': false,
          'isModule': false,
          'length': 160,
          'scriptLanguage': "JavaScript",
          'embedderName': `http://127.0.0.1/${element}`
        }
        this.fireEventWithTag('Debugger.scriptParsed', o)
      }
    }
  }

  firePausedEvent(data) {
    const { res, t, callFrame } = data
    const obj = res
    const itemId = this.runtime.setItem(obj)
    obj.__objectId__ = `${itemId}_0`
    this.runtime.setObjectId(obj, itemId)
    const asyncStackTrace = []
    for (let index = 0; index < callFrame.length; index++) {
      const element = callFrame[index]
      asyncStackTrace.push({
        "functionName": element,
        "scriptId": t.start.scriptId,
        "url": "http://49.232.147.237/aa/output.js",
        "lineNumber": 0,
        "columnNumber": 0
      })
    }
    const scopeChain = [
      {
        'type': "local",
        'object': {
          'type': "object",
          'className': "Object",
          'description': "Object",
          'objectId': obj.__objectId__
        },
        'startLocation': t.start,
        'endLocation': t.end
      }
    ]
    const result = {
      asyncStackTrace: {
        callFrames: asyncStackTrace
      },
      'callFrames': [
        {
          'callFrameId': "2664138649130159018.65.0",
          'functionName': "",
          "functionLocation": {
            "scriptId": t.start.scriptId,
            "lineNumber": t.start.lineNumber,
            "columnNumber": t.start.columnNumber
          },
          "location": {
            "scriptId": t.start.scriptId,
            "lineNumber": t.start.lineNumber,
            "columnNumber": t.start.columnNumber
          },
          'url': "http://49.232.147.237/aa/output.js",
          'scopeChain': scopeChain
        }
      ],
      'reason': "other"
    }
    this.fireEvent('Debugger.paused', result)
    // const sdkData = JSON.parse(JSON.stringify(obj))
    // this.setSdkInterruptObj(sdkData)
    this.runtime.status = true
    window.ServiceJSBridge.invoke('remoteDebuggerInterrupt', { interrupt: true })
  }

  fireResumedEvent () {
    this.fireEvent('Debugger.resumed')
  }

  /**
   * 将debug需要的数据格式化并返回给sdk
   * @param {*} data
   */
  setSdkInterruptObj (data) {
    for (const key in data) {
      if (Object.hasOwnProperty.call(data, key)) {
        if (key === '__objectId__') {
          break
        }
        const element = data[key]
        const __objectId__ = data.__objectId__
        const type = typeof element
        const result = this.runtime.getItem(__objectId__)
        const res = util.EvaluatePropertiesRes(result)
        data.__finclip_result__ = res
        if (type === 'object') {
          this.setSdkInterruptObj(element)
        }
      }
    }
  }

  getScriptIdByUrl (url) {
    if (__wxConfig__ && __wxConfig__.__scriptIds__) {
      const __scriptIds__ = __wxConfig__.__scriptIds__
      for (const key in __scriptIds__) {
        const element = __scriptIds__[key]
        if (url.endsWith(element)) {
          return key
        }
      }
    }
  }

  getScriptLinesByScripId (id) {
    let lines = []
    if (__wxConfig__ && __wxConfig__.__scriptLines__) {
      const obj = __wxConfig__.__scriptLines__
      for (const key in obj) {
        if (key === id) {
          lines = obj[key].lines
        }
      }
    }
    return lines
  }

  debuggerFirePausedEvent (data) {
    this.firePausedEvent(data)
  }
}

export default Debugger
